// PIC18F24K50 Analog Joystick HID USB Device
// V. 0.4
// For e.g. Genge&Thoma analog joystick
// Compliled with MPLAB 8.92 and CCS PCH 5.021
//
// 30.9.2012 v. 0.3: Adapted to dual_atari_vertical PCB
// Pinout still original (for d-sub), perhaps not optimal since
// +5V for pots must be supplied by PIC output pin
//
// 11.1.2015 v. 0.4: Adapted to PIC18F24K50 (jakadapter v2.1 HW)


#include <18F24K50.h>
#device ADC=8
#build(stack=256)
#fuses PLL3X,INTRC,NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV28,NOPROTECT,NOLVP,NODEBUG,PLLEN,NOCPUDIV,PUT,MCLR
#use delay(int=16MHz, clock=48MHz, USB_FULL, act=USB)

#include <pic18_usb.h>   //Microchip PIC18Fxx5x hardware layer for usb.c
#include <usb_desc_jmouse.h>    //USB Configuration and Device descriptors for this UBS device
#include <usb.c>        //handles usb setup tokens and get descriptor reports
#include "jakadap21_pindef.h"	// Pin definitions for dual atari v2.1 hw
#include "usb_bootloader.h"	// sets program location and reset vector for use with bootloader

void main(void) {
	int a;
	signed int8 out_data[4];
	signed int16 temp;
	unsigned int8 x_neutral;
	unsigned int8 y_neutral;
	
	PORT_E_PULLUPS(TRUE);	// Activates weak pull-up on pin RE3 (VPP/MCLR)
	PORT_B_PULLUPS(0b11000011);  // Activates weak pull-up on RB0,RB1,RB6,RB7

	OUTPUT_HIGH(X1_7);			// Activates +5V to joystick 1 pin 7
	OUTPUT_HIGH(X1_7_ADD);		// Activates additional pin for more current
	OUTPUT_HIGH(X2_7);			// Activates +5V to joystick 2 pin 7
	OUTPUT_HIGH(X2_7_ADD);		// Activates additional pin for more current
	
	// drive unused pins to GND:
	OUTPUT_LOW(UNUSED1);
	OUTPUT_LOW(UNUSED2);
	OUTPUT_LOW(UNUSED3);
	#ifdef UNUSED4
	OUTPUT_LOW(UNUSED4);
	#endif

	setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL|X2_9_ANSEL );
	setup_adc(ADC_CLOCK_DIV_64); // Only recommended ADC divisor for 48 MHz Fosc. Equals TAD period of 1.3 s

	// Kalibroidaan x-akselin neutral kun usb kytketn
	// Jos tikkua knt kun kytkee usbiin, hiiri j 'vetmn'
	set_adc_channel (X1_9_AN);	// POT_X
	delay_ms(1);	
	read_adc(ADC_START_ONLY);
	while(!(adc_done())) {    
	} 
	x_neutral = read_adc(ADC_READ_ONLY);
	
	// Y-akselin neutral
	set_adc_channel (X1_5_AN);	// POT_Y
	delay_ms(1);	
	read_adc(ADC_START_ONLY);
	while(!(adc_done())) {    
	} 
	y_neutral = read_adc(ADC_READ_ONLY);
	
   usb_init_cs();

   while (TRUE) {
      usb_task();
      if (usb_enumerated()) {
         if (usb_put_packet(1,out_data,4,USB_DTS_TOGGLE))
       	   a=0;
           if(!input(X1_6)){	// Vasen nappi
               a=1;      
           } 
           if(!input(X1_4)){ // Oikea nappi
               a=a+2;      
           }
			out_data[0]=a; //button state goes here
           
			set_adc_channel (X1_9_AN);	// POT_X
			delay_ms(1);	
			read_adc(ADC_START_ONLY);
			while(!(adc_done())) {    
			} 
			temp = read_adc(ADC_READ_ONLY);
			temp = x_neutral - temp;
			temp = temp / 6;
			if (temp<-127)			// Hanskataan yli- ja alivuodot
				out_data[1] = -127;
			else if (temp>127)
				out_data[1] = 127;
			else
				out_data[1] = temp;

			set_adc_channel (X1_5_AN);	// POT_Y
			delay_ms(1);	
			read_adc(ADC_START_ONLY);
			while(!(adc_done())) {    
			} 
			temp = read_adc(ADC_READ_ONLY);
			temp = temp - y_neutral;
			temp = temp / 6;
			if (temp<-127)
				out_data[2] = -127;
			else if (temp>127)
				out_data[2] = 127;
			else
				out_data[2] = temp;

			out_data[3] = 0; // wheel state

         delay_ms(1);
      }
   }
}
